import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  scaleOrdinal,
  schemeCategory10
} from 'd3';
import { colorLegend } from './colorLegend';
import { loadData } from './loadData';
import { scatterplot } from './scatterplot';
import { dropdownMenu } from './dropdownMenu';

const colorScale = scaleOrdinal();
const colorValue = d => d.artist;

var artists;
var selectedArtist;
var years = [];
var selectedYear;
var data;

const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

const title = 'Top 10 Artists, Total Plays per Month';

const colorLegendG = svg.append('g')
  .attr('transform', `translate(20, 70)`);

loadData().then(processedData => {
  data = processedData;
  
  // Sort artist list by total plays
  artists = [...new Set(data.map(d => d.artist))];
  var artistPlays = {};
  artists.forEach(a => { artistPlays[a] = 0; });
  data.forEach(d => { artistPlays[d.artist] += d.plays; });

  //https://stackoverflow.com/questions/1069666/sorting-javascript-object-by-property-value
  var sortable = [];
  for (var artist in artistPlays) {
      sortable.push([artist, artistPlays[artist]]);
  }
  sortable.sort((a, b) => { return b[1] - a[1]; });
  artists = sortable.map(d => d[0]);
  //console.log(artists);
  //console.log(sortable);
  selectedArtist = artists[0];
  
  //Also make a list of years
  let startYear;
  let endYear;
  [startYear, endYear] = extent(data, d => d.month.getFullYear())
  for (var i = startYear; i <= endYear; i++)
    years.push(i);
  
  selectedYear = years[0];
  render();
});


const onClickArtist = d => {
  selectedArtist = d;
  render();
};

const onClickYear = d => {
  selectedYear = +d;
  render();
};


const render = () => {
  select('#artist-menu')
    .call(dropdownMenu, {
      options: artists,
      onOptionClicked: onClickArtist,
      selectedOption: selectedArtist
    });
  
  
  select('#year-menu')
    .call(dropdownMenu, {
      options: years,
      onOptionClicked: onClickYear,
      selectedOption: selectedYear
    });
  
  colorScale
    .domain(artists)
    .range(schemeCategory10); 
  
  colorLegendG.call(colorLegend, {
    colorScale,
    circleRadius: 8,
    spacing: 20,
    textOffset: 12,
    backgroundRectWidth: 135,
    onClickArtist,
    selectedArtist
  });
  
  var filteredData = []; 
  if (selectedYear)
  {
    filteredData = data.filter(d => {
      //console.log(d.month.getFullYear())
      return d.month.getFullYear() === selectedYear;
    });
  }
  else
  {
    filteredData = data;
    console.log('regular data');
  }
  
  svg.call(scatterplot, {
    colorScale,
    colorValue,
    selectedArtist,
    selectedYear,
    circleRadius: 3,
    margin: { top: 60, right: 30, bottom: 88, left: 210 },
    width,
    height,
    filteredData,
    title
  });
};