// Most of the axis code borrowed from the example:
// https://vizhub.com/curran/98ba4daacc92442f8d9fd7d91bfd712a

import { 
  select, 
  scaleTime, 
  scaleLinear, 
  extent,
  axisLeft,
  axisBottom,
  format,
  timeFormat
} from 'd3';

export const scatterplot = (selection, props) => {
  const {
    colorScale,
    colorValue,
    selectedArtist,
    selectedYear,
    circleRadius,
    margin,
    width,
    height,
    filteredData,
    title
  } = props;
  
  var data = filteredData;
  
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;		 
  const xValue = d => d.month;
  const xAxisLabel = 'Time';
 
  const yValue = d => d.plays;
  const yAxisLabel = 'Plays';
  
  const xScale = scaleTime()
    .domain([
      new Date(selectedYear, 0),
      new Date(selectedYear, 11)
    ])
    .range([0, innerWidth])
    .nice();
  
  
  console.log(data)
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
  
  selection.append('text')
    .attr('class', 'title')
    .attr('x', margin.left + innerWidth / 2)
    .attr('y', 45)
    .text(title); 

  
  const g = selection.selectAll('.container').data([null]);
  const gEnter = g
  	.enter().append('g')
      .attr('class', 'container');
  gEnter
  	.merge(g)
      .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxis = axisBottom(xScale)
    .ticks(12)
    .tickSize(-innerHeight)
    .tickPadding(15)
    .tickFormat(timeFormat('%B'));
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(5)
    .ticks(8)
    .tickFormat(format('.0f')); 
  
  const yAxisG = g.select('.y-axis');
  const yAxisGEnter = gEnter
    .append('g')
      .attr('class', 'y-axis');
  yAxisG
    .merge(yAxisGEnter)
      .call(yAxis)
      .selectAll('.domain').remove();
  
  const yAxisLabelText = yAxisGEnter
    .append('text')
      .attr('class', 'axis-label')
      .attr('y', -35)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
    .merge(yAxisG.select('.axis-label'))
      .attr('x', -innerHeight / 2)
      .text(yAxisLabel);
  
  const xAxisG = g.select('.x-axis');
  const xAxisGEnter = gEnter
    .append('g')
      .attr('class', 'x-axis');
  xAxisG
    .merge(xAxisGEnter)
      .attr('transform', `translate(0,${innerHeight})`)
      .call(xAxis)
      .selectAll('.domain').remove();
  
  const xAxisLabelText = xAxisGEnter
    .append('text')
      .attr('class', 'axis-label')
      .attr('y', 55)
      .attr('fill', 'black')
    .merge(xAxisG.select('.axis-label'))
      .attr('x', innerWidth / 2)
      .text(xAxisLabel);
  
  const points = g.merge(gEnter)
    .selectAll('.artist').data(data);
  points
    .enter().append('circle')
      .attr('class', 'artist')
      .attr('r', circleRadius)
      .attr('fill', d => colorScale(d.artist))
     .merge(points)
  	 .transition()
  		.duration(2000)
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('opacity', d =>
        (!selectedArtist || d.artist === selectedArtist) ? 1 : 0);
};